@@ -39,10 +39,7 @@
 #ifndef nsUnicodeToCNS11643p2_h___
 #define nsUnicodeToCNS11643p2_h___
 
-#include "nsUCvTW2Support.h"
-
-//----------------------------------------------------------------------
-// Class nsUnicodeToCNS11643p2 [declaration]
+#include "nsISupports.h"
 
 /**
  * A character set converter from Unicode to CNS11643p2.
@@ -50,22 +47,8 @@
  * @created         06/Apr/1999
  * @author  Catalin Rotaru [CATA]
  */
-class nsUnicodeToCNS11643p2 : public nsTableEncoderSupport
-{
-public:
-
-  /**
-   * Class constructor.
-   */
-  nsUnicodeToCNS11643p2();
-
-protected:
-
-  //--------------------------------------------------------------------
-  // Subclassing of nsEncoderSupport class [declaration]
-
-  NS_IMETHOD GetMaxLength(const PRUnichar * aSrc, PRInt32 aSrcLength, 
-      PRInt32 * aDestLength);
-};
+NS_METHOD
+nsUnicodeToCNS11643p2Constructor(nsISupports *aOuter, REFNSIID aIID,
+                            void **aResult);
 
 #endif /* nsUnicodeToCNS11643p2_h___ */
