@@ -1,39 +1,44 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code, released
  * March 31, 1998.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   John Bandhauer <jband@netscape.com> (original author)
  *   Pierre Phaneuf <pp@ludusdesign.com>
  *   Mike Shaver <shaver@mozilla.org>
  *
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU Public License (the "GPL"), in which case the
- * provisions of the GPL are applicable instead of those above.
- * If you wish to allow use of your version of this file only
- * under the terms of the GPL and not to allow others to use your
- * version of this file under the NPL, indicate your decision by
- * deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL.  If you do not delete
- * the provisions above, a recipient may use your version of this
- * file under either the NPL or the GPL.
- */
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 /* Data conversion between native and JavaScript types. */
 
@@ -704,18 +709,14 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
 
             if(useAllocator)
             {
-                if(str)
+                // XXX extra string copy when isNewString
+                if(str && !isNewString)
                 {
                     XPCReadableJSStringWrapper *wrapper =
                         XPCStringConvert::JSStringToReadable(str);
                     if(!wrapper)
                         return JS_FALSE;
 
-                    // Ask for the shared buffer handle, which will root the
-                    // string.
-                    if(isNewString && ! wrapper->GetSharedBufferHandle())
-                        return JS_FALSE;
-
                     *((const nsAString**)d) = wrapper;
                 }
                 else if(JSVAL_IS_NULL(s))
@@ -729,7 +730,8 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                 }
                 else
                 {
-                    const nsAString *rs = new nsAutoString(chars, length);
+                    // use nsString to encourage sharing
+                    const nsAString *rs = new nsString(chars, length);
                     if(!rs)
                         return JS_FALSE;
                     *((const nsAString**)d) = rs;
@@ -878,23 +880,22 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
 
             length = JS_GetStringLength(str);
 
+            nsCString *rs;
             if(useAllocator)
             {                
-                const nsACString *rs = new NS_ConvertUCS2toUTF8((const PRUnichar*)chars, length);
+                // Use nsCString to enable sharing
+                rs = new nsCString();
                 if(!rs)
                     return JS_FALSE;
 
-                *((const nsACString**)d) = rs;
+                *((const nsCString**)d) = rs;
             }
             else
             {
-                nsCString* rs = *((nsCString**)d);
-
-                // XXX This code needs to change when Jag lands the new
-                // UTF8String implementation.  Adopt() is a method that 
-                // shouldn't be used by string consumers.
-                rs->Adopt(ToNewUTF8String(nsDependentString((const PRUnichar*)chars, length)));
+                rs = *((nsCString**)d);
             }
+            CopyUTF16toUTF8(nsDependentString((const PRUnichar*)chars, length),
+                            *rs);
             return JS_TRUE;
         }
 
